#! /usr/bin/env perl

use Test::More;
use Test::Exception;
use Sub::Name;
use lib 'lib';
use DB::Test;
use Modern::Perl;
use Data::Printer alias => 'Dumper';

use Test::Schema;

my ($dsn, $user, $pass) = @ENV{map { "DBICTEST_PG_${_}" } qw/DSN USER PASS/};

plan skip_all => <<'EOM' unless $dsn && $user;
Set $ENV{DBICTEST_PG_DSN}, _USER and _PASS to run this test
( NOTE: This test drops and create a table called 'this_schema.item_values
as following schema: 'this_schema'
EOM

### load any test classes that are defined further down in the file via BEGIN blocks

our @test_classes; #< array that will be pushed into by test classes defined in this file

my $schema;

$schema = DB::Test->new(
  dbi => $dsn, 
  uid => $user, 
  pass => $pass
)->schema;

drop_test_schema($schema);
create_test_schema($schema);

my $rs = $schema->resultset('This::Schema::Item_Values');

for (1..6) {
  isa_ok( $rs->create({ item => 'Item ' . $_ }), 'DBIx::Class::Row' )
}

undef $schema;

$schema = DB::Test->new(
  dbi => $dsn, 
  uid => $user, 
  pass => $pass
)->schema;

$rs = $schema->resultset('This::Schema::Item_Values');

my $it = $rs->search( 
    {
      item => 'Item 5',
    },
);

isa_ok( $it, 'DBIx::Class::ResultSet' );

my $first = $it->first;

like( $first->get_column('item'), qr/^Item.*$/, 'search on resultset OK');

ok( $it->update( { secondary => 1 } ) == 1, 'test of single query bulk update' );


done_testing;

END {
    return unless $schema;
    drop_test_schema($schema);
    undef $schema;
};


######### SUBROUTINES

sub create_test_schema {
    my $schema = shift;
    $schema->storage->dbh_do(sub {
      my (undef,$dbh) = @_;

      local $dbh->{Warn} = 0;

      my $std_item_table = <<EOS;
(
  item VARCHAR(9) NOT NULL PRIMARY KEY,
  secondary BOOLEAN
)
EOS

      $dbh->do("CREATE SCHEMA this_schema");
      $dbh->do("CREATE TABLE this_schema.item_values $std_item_table");

      $dbh->do('set search_path=this_schema,public');
  });
}

sub drop_test_schema {
    my ( $schema, $warn_exceptions ) = @_;

    $schema->storage->dbh_do(sub {
        my (undef,$dbh) = @_;

        local $dbh->{Warn} = 0;

        for my $stat (
                      'DROP SCHEMA this_schema CASCADE',
                     ) {
            eval { $dbh->do ($stat) };
            diag $@ if $@ && $warn_exceptions;
        }
    });
}

